searchData={"content_type":"text/markdown","items":[{"doc":"This module is a stand-in for reading a line from files into a tuple of\nbinaries.\n\nThis module exists to provide default behavior and a reference implementation\nfor the `Bio.Sequential` behaviour of the `bio_ex_sequences` package.","ref":"Bio.IO.SequenceTuple.html","title":"Bio.IO.SequenceTuple","type":"module"},{"doc":"Generate a tuple of label and sequence from reading fasta and fasta like\nformats:\n\n    { ,  }\n\n    >header1\n    tagctag\n\n    {\"header1\", \"tagctag\"}","ref":"Bio.IO.SequenceTuple.html#new/2","title":"Bio.IO.SequenceTuple.new/2","type":"function"},{"doc":"A struct representing the quality score of a FASTQ file.\n\n``` elixir\n%QualityScore{\n  scoring_characters: binary(),\n  scores: [integer()],\n  label: binary(),\n  encoding: Bio.IO.FastQ.quality_encoding()\n}\n```","ref":"Bio.QualityScore.html","title":"Bio.QualityScore","type":"module"},{"doc":"Allow the input of FASTQ formatted files.\n\nThis implementation references the documentation from\n[NCBI](https://www.ncbi.nlm.nih.gov/sra/docs/submitformats/#fastq-files) and\nuses the Phred scoring 33 offset by default when reading quality scores.","ref":"Bio.IO.FastQ.html","title":"Bio.IO.FastQ","type":"module"},{"doc":"Read a FASTQ formatted file into memory\n\nThe `read/2` function returns an error tuple of the content or error code from\n`File.read`. You can use `:file.format_error/1` to get a descriptive string of\nthe error.\n\nContent is returned as a list of tuples where the first element is a struct of\nthe type from the `type` option, and the second element is a\n`Bio.QualityScore`.","ref":"Bio.IO.FastQ.html#read/2","title":"Bio.IO.FastQ.read/2","type":"function"},{"doc":"- `type` - The module for the Sequence type that you want the returned value\nin. Defaults to `Bio.IO.SequenceTuple`. Module should implement the `Bio.Sequential`\nbehaviour or minimally expose a `new/2` function which is parametrically\nisomorphic.\n- `quality_encoding` - Determines the encoding of the quality scores.","ref":"Bio.IO.FastQ.html#read/2-options","title":"Options - Bio.IO.FastQ.read/2","type":"function"},{"doc":"Read a FASTQ formatted file\n\nThe same as `read/2`, but will raise a `File.Error` on failure.","ref":"Bio.IO.FastQ.html#read!/2","title":"Bio.IO.FastQ.read!/2","type":"function"},{"doc":"","ref":"Bio.IO.FastQ.html#t:quality_encoding/0","title":"Bio.IO.FastQ.quality_encoding/0","type":"type"},{"doc":"","ref":"Bio.IO.FastQ.html#t:read_opts/0","title":"Bio.IO.FastQ.read_opts/0","type":"type"},{"doc":"Allow the input/output of FASTA formatted files.\n\nThe FASTA file format is composed of pairs of lines where the pair is\ndemarcated by the \">\" character. All data proceeding the \">\" character\nrepresents the 'header' of the pair, while the next line after a newline\nrepresents sequence data.\n\nAny data after subsequent newlines that are _not_ preceded by a second \">\"\ncharacter are assumed to be multi-line data. For example, the following two\nfiles would be considered equivalent data:\n\n```\n# fasta 1\n>header1\natgcatgca\n```\n\nand\n\n```\n# fasta 2\n>header1\natgc\natgca\n```\n\nThe FASTA file format does not specify the type of the data in the sequence.\nThat means that you can reasonably store RNA, DNA, amino acid, or any other\nsequence using the format. The expectation is that the data is ASCII encoded.\n\nThe methods in this module do support reading into specified types. See\n`read/2` for more details.","ref":"Bio.IO.Fasta.html","title":"Bio.IO.Fasta","type":"module"},{"doc":"Produces the same output as `read!/2`, but presumes that the file contents are\nloaded into a binary.","ref":"Bio.IO.Fasta.html#from_binary/2","title":"Bio.IO.Fasta.from_binary/2","type":"function"},{"doc":"Read a FASTA formatted file\n\nThe `read/2` function returns an error tuple of the content or error code from\n`File.read`. You can use `:file.format_error/1` to get a descriptive string of\nthe error.\n\nYou can specify the return type of the contents by using a module which\nimplements the `Bio.Sequential` or equivalent behaviour. Specifically the type\nmust have a `new/2` function for building the struct.\n\nThe default option for the reader is a special module `Bio.IO.SequenceTuple`, which\nwill return the label and sequence as a tuple of raw binary. See\n`Bio.IO.SequenceTuple.new/2` for details.","ref":"Bio.IO.Fasta.html#read/2","title":"Bio.IO.Fasta.read/2","type":"function"},{"doc":"- `:type` - The module for the type of struct you wish to have returned. This\nshould minimally implement a `new/2` function equivalent to the\n`Bio.Sequential` behaviour. Otherwise the base `Bio.IO.SequenceTuple` is used.\n- `:parse_header` - A callable for parsing the header values of the FASTA\nfile. Otherwise identity is used and the header is returned as is.","ref":"Bio.IO.Fasta.html#read/2-options","title":"Options - Bio.IO.Fasta.read/2","type":"function"},{"doc":"iex>Bio.IO.Fasta.read(\"test/fasta/test_1.fasta\")\n  {:ok, [{\"header1\", \"ataatatgatagtagatagatagtcctatga\"}]}","ref":"Bio.IO.Fasta.html#read/2-examples","title":"Examples - Bio.IO.Fasta.read/2","type":"function"},{"doc":"Read a FASTA formatted file\n\nThe same as `read/2`, but will raise a `File.Error` on failure.","ref":"Bio.IO.Fasta.html#read!/2","title":"Bio.IO.Fasta.read!/2","type":"function"},{"doc":"Write a FASTA file using sequence data.\n\nThe data type that this function accepts is varied, and may be one of a number\nof `List`s. Examples of which types are handled:\n\nList:\n``` elixir\n  # a list of header/sequence tuples\n  [{header(), sequence()}, ...]\n  # a list of header/sequence implicitly paired\n  [header(), sequence(), header(), sequence(), ...]\n  # a list of struct()\n  [%Bio.Sequence._{}, ...]\n```\n\nWhere `%Bio.Sequence._{}` indicates any struct of the `Bio.Sequence` module or\nmodules implementing the `Bio.Sequential` behaviour.\n\nIt also supports data in a `Map` format:\n\n``` elixir\n%{\n  headers: [header(), ...],\n  sequences: [sequence(), ...]\n}\n```","ref":"Bio.IO.Fasta.html#write/3","title":"Bio.IO.Fasta.write/3","type":"function"},{"doc":"iex> Fasta.write(\"/tmp/test_file.fasta\", [\"header\", \"sequence\", \"header2\", \"sequence2\"])\n    :ok\n\nWill return error types in common with `File.write/3`","ref":"Bio.IO.Fasta.html#write/3-examples","title":"Examples - Bio.IO.Fasta.write/3","type":"function"},{"doc":"","ref":"Bio.IO.Fasta.html#t:fasta_data/0","title":"Bio.IO.Fasta.fasta_data/0","type":"type"},{"doc":"","ref":"Bio.IO.Fasta.html#t:header/0","title":"Bio.IO.Fasta.header/0","type":"type"},{"doc":"","ref":"Bio.IO.Fasta.html#t:read_opts/0","title":"Bio.IO.Fasta.read_opts/0","type":"type"},{"doc":"","ref":"Bio.IO.Fasta.html#t:sequence/0","title":"Bio.IO.Fasta.sequence/0","type":"type"},{"doc":"Read a SnapGene file\n\nThe file is read into a struct with the following fields:\n\n``` elixir\n%Bio.IO.SnapGene{\n    sequence: %Bio.Sequence.DnaStrand{},\n    circular?: boolean(),\n    valid?: boolean(),\n    features: tuple()\n  }\n```\n\nThe `circular?` and `sequence` fields are parsed from the DNA packet.\n\nThe `sequence` field is represented by default as a `Bio.Sequence.DnaStrand`,\nbut any module that behaves as a `Bio.Behaviours.Sequence` can be used, since\nthe `new/2` method is applied to create the struct.\n\n> ##","ref":"Bio.IO.SnapGene.html","title":"Bio.IO.SnapGene","type":"module"},{"doc":">\n> No validation is applied to the sequence, so you can force an invalid sequence\n> struct by passing a module for the incorrect sequence.\n\nThe `valid?` property is determined by parsing the SnapGene cookie to ensure that it\ncontains the requisite \"SnapGene\" string.\n\n> ##","ref":"Bio.IO.SnapGene.html#module-error-error","title":"Error {: .error} - Bio.IO.SnapGene","type":"module"},{"doc":">\n> The concept of validity has to do with the snap gene file, and not the\n> sequence or any other of the parsed data.\n\nFeatures require a bit more explanation, since they are stored in XML. Parsing\nthem into a map is certainly a possibility, but it seemed like doing so would\nreduce the ability of a developer to leverage what I am hoping is a lower\nlevel library than some.\n\nIn the interest of leaving the end user with as much power as possible, this\nmethod does not attempt to parse the XML stored within the file. Instead, the\nXML is returned to you in the form generated by `:xmerl_scan.string/1`. In\ndoing it this way you have access to the entire space of data stored within\nthe file, not just a subset that is parsed. This also means that in order to\nquery the data, you need to be comfortable composing XPaths. As an example, if\nyou have a terminator feature as the first feature and you want to get the\nsegment range:\n\n    iex>{:ok, sample} = SnapGene.read(\"test/snap_gene/sample-e.dna\")\n    ...>:xmerl_xpath.string('string(/*/Feature[1]/Segment/@range)', sample.features)\n    {:xmlObj, :string, '400-750'}\n\nAs another note, this will also require some familiarity with the file type,\nfor example whether or not a range is exclusive or inclusive on either end.\nAttempting to access a node that doesn't exist will return an empty array.\n\n    iex>{:ok, sample} = SnapGene.read(\"test/snap_gene/sample-e.dna\")\n    ...>:xmerl_xpath.string('string(/*/Feature[1]/Unknown/Path/@range)', sample.features)\n    {:xmlObj, :string, []}\n\nThe semantics of this are admittedly odd. But there's not much to be done\nabout that.\n\nThe object returned from `:xmerl_xpath.string/[2,3,4]` is a tuple, so\n`Enumerable` isn't implemented for it. You're best off sticking to XPath to\nget the required elements. The counts of things are simple enough to retrieve\nin this way though. For example, if I wanted to know how many Feature Segments\nthere were:\n\n    iex>{:ok, sample} = SnapGene.read(\"test/snap_gene/sample-e.dna\")\n    ...>:xmerl_xpath.string('count(/*/Feature/Segment)', sample.features)\n    {:xmlObj, :number, 2}\n\nNow it's a simple matter to map over the desired queries to build up some data\nfrom the XML:\n\n    iex>{:ok, sample} = SnapGene.read(\"test/snap_gene/sample-e.dna\")\n    ...>Enum.map(1..2, fn i -> :xmerl_xpath.string('string(/*/Feature[#{i}]/Segment/@range)', sample.features) end)\n    [{:xmlObj, :string, '400-750'},{:xmlObj, :string, '161-241'}]\n\nI cover the basics of using XPath to perform queries in the [Using\nXML](guides/howtos/use_xml_and_xpath.md) guide. I also plan to write a follow\nup guide with further examples of queries, and an explanation of the mapping\nof concepts between the XML and what is parsed from BioPython.","ref":"Bio.IO.SnapGene.html#module-note-neutral","title":"Note {: .neutral} - Bio.IO.SnapGene","type":"module"},{"doc":"Read the contents of a SnapGene file.\n\nTakes a filename and reads the contents into the `%Bio.IO.SnapGene{}` struct.\nReturns an error tuple on failure with the cause from `File.read/1`.\n\nYou can use `:file.format_error/1` to get a descriptive string of the error.","ref":"Bio.IO.SnapGene.html#read/2","title":"Bio.IO.SnapGene.read/2","type":"function"},{"doc":"# How To Work With XML","ref":"use_xml_and_xpath.html","title":"How To Work With XML","type":"extras"},{"doc":"This is an overview of how to use XPaths to query an XML structure in Elixir.","ref":"use_xml_and_xpath.html#document-purpose","title":"Document Purpose - How To Work With XML","type":"extras"},{"doc":"I'm a software engineer. Plain and simple. I do not know what you want, nor do I\nhave a desire to lock you into a specific sub-set of the data that can be\nretrieved from any given format. This is no exception.\n\nMy primary goal in this project is to provide powerful, efficient, and\nexpressive tooling, which will allow more intuitive tooling in higher level\nprojects.","ref":"use_xml_and_xpath.html#why-force-you-to-deal-with-xpath","title":"Why force you to deal with XPath? - How To Work With XML","type":"extras"},{"doc":"Wikipedia's definition of XML is fairly useful here:\n\n> Extensible Markup Language (XML) is a markup language and file format for storing,\n> transmitting, and reconstructing arbitrary data. It defines a set of rules for\n> encoding documents in a format that is both human-readable and machine-readable.\n> The World Wide Web Consortium's XML 1.0 Specification of 1998 and several other\n> related specifications—all of them free open standards—define XML.\n> That is, XML is a structured document format. This allows us to treat the\n> document according the Document Object Model (DOM). This will be familiar to\n> anyone with experience in developing for the web, as HTML is a similar format.\n> In fact, you can use XPath to query HTML documents in JavaScript as well.\n\nThe most basic form of access is the root. All well formed documents will have a\nroot node against which you can query. This will often take the form of the `/`\ncharacter. This is a very common idiom for tree-like structures, not limited to\nthe DOM. Even the root directory of your file-system (on \\*nix) will be\nrepresented as such.\n\nEach element beyond the root is considered as a child. You can access them with\n`/ChildNode`. For those familiar with HTML, the biggest difference between XML\nand HTML is that there isn't a standard for the nodes. In XML, each node can be\ndefined however you would like. So this is a perfectly valid document:\n\n```xml\n \n \n   \n     Content \n     Content \n     Content \n   \n   Content \n   Content \n \n```\n\nWhere the root node is `/OuterBody`, and the children are `OuterBody/InnerBody`,\n`OuterBody/OuterField1`, and `OuterBody/OuterField2`.\n\nThe fields do _not_ need to be numbered though, and more often they won't be.\nSomething more common might be:\n\n```xml\n \n \n   \n     Content \n     Content \n     Content \n   \n   Content \n   Content \n \n```\n\nThe fields will normally be named according to the semantic meaning of the\ncontents. You can also have various attributes that are associated with each\nnode:\n\n```xml\n \n \n   \n     Content \n     Content \n     Content \n   \n   Content \n   Content \n \n```\n\nThis is useful for when you are representing hierarchical relationships. These\nare access by using the `@ ` notation. So the above would be\n`/Outerbody/InnerBody/@name`.","ref":"use_xml_and_xpath.html#xpath-basics","title":"XPath Basics - How To Work With XML","type":"extras"},{"doc":"Querying paths with Erlang is very straightforward. The biggest issue with doing\nso is that you will need to convert between binaries and character lists. As\nthis interface is elixir, the expectation is character list.\n\n```elixir\n:xmerl_xpath.string('/path/to/node', xml_doc)\n```\n\nLet's use an example that is a little easier to think about, something like\nresearch paper citations:\n\n```xml\n \n \n   \n     A Novel Method for Quantification of Midi-Chlorians From Salivary Samples \n     \n        \n        \n        \n     \n   \n   \n     Harmonic Crystal Frequencies and Light-Saber Characteristics: Review \n     \n        \n        \n        \n     \n   \n   \n     Modification of the Natural Foci for Midi-Chlorian Energies \n     \n        \n        \n        \n        \n     \n   \n \n```\n\n**Note**: I had to look up a lot of names and general themes on Wookipedia. I\nhope that you can enjoy the references for what they are. I tried.\n\nThere are obviously different ways that you might want to query this data. For\nexample, let's say that you wanted to get the paper with `refNum=\"2\"`. I'll\nstart by showing how to parse the file, assuming you call it something like\n`practice.xml`\n\n```elixir\n{xml, _rest} = :xmerl_scan.file('practice.xml')\n```\n\nDemonstrating again that when interacting with the Erlang libraries, you should\nrecall to use character lists not binaries. That said, if you _do_ use a binary\nhere it won't actually fail.\n\nNow to perform the query:\n\n```elixir\n:xmerl_xpath.string('/*/Paper[@refNum=\"2\"]', xml)\n```\n\nThis will net you back the slightly massive looking:\n\n```elixir\n[\n  {:xmlElement, :Paper, :Paper, [], {:xmlNamespace, [], []}, [References: 1], 4,\n   [\n     {:xmlAttribute, :refNum, [], [], [], [Paper: 4, References: 1], 1, [], '2',\n      false}\n   ],\n   [\n     {:xmlText, [Paper: 4, References: 1], 1, [], '\\n    ', :text},\n     {:xmlElement, :Title, :Title, [], {:xmlNamespace, [], []},\n      [Paper: 4, References: 1], 2, [],\n      [\n        {:xmlText, [Title: 2, Paper: 4, References: 1], 1, [],\n         'Harmonic Crystal Frequencies and Light-Saber Characteristics: Review',\n         :text}\n      ], [], \".\", :undeclared},\n     {:xmlText, [Paper: 4, References: 1], 3, [], '\\n    ', :text},\n     {:xmlElement, :Authors, :Authors, [], {:xmlNamespace, [], []},\n      [Paper: 4, References: 1], 4, [],\n      [\n        {:xmlText, [Authors: 4, Paper: 4, References: 1], 1, [], '\\n      ',\n         :text},\n        {:xmlElement, :Author, :Author, [], {:xmlNamespace, [], []},\n         [Authors: 4, Paper: 4, References: 1], 2,\n         [\n           {:xmlAttribute, :name, [], [], [],\n            [Author: 2, Authors: 4, Paper: 4, References: 1], 1, [],\n            'A. Skywalker', false}\n         ], [], [], \".\", :undeclared},\n        {:xmlText, [Authors: 4, Paper: 4, References: 1], 3, [], '\\n      ',\n         :text},\n        {:xmlElement, :Author, :Author, [], {:xmlNamespace, [], []},\n         [Authors: 4, Paper: 4, References: 1], 4,\n         [\n           {:xmlAttribute, :name, [], [], [],\n            [Author: 4, Authors: 4, Paper: 4, References: 1], 1, [],\n            'O. Kenobi', false}\n         ], [], [], \".\", :undeclared},\n        {:xmlText, [Authors: 4, Paper: 4, References: 1], 5, [], '\\n      ',\n         :text},\n        {:xmlElement, :Author, :Author, [], {:xmlNamespace, [], []},\n         [Authors: 4, Paper: 4, References: 1], 6,\n         [\n           {:xmlAttribute, :name, [], [], [],\n            [Author: 6, Authors: 4, Paper: 4, References: 1], 1, [], 'A. Tano',\n            false}\n         ], [], [], \".\", :undeclared},\n        {:xmlText, [Authors: 4, Paper: 4, References: 1], 7, [], '\\n    ',\n         :text}\n      ], [], \".\", :undeclared},\n     {:xmlText, [Paper: 4, References: 1], 5, [], '\\n  ', :text}\n   ], [], \".\", :undeclared}\n]\n```\n\nThis is a slightly intimidating wall of data. Within it, you can see the\npieces that matter to you. We have the title of the paper, as well as the\nauthors. This is a good start, but what if you want a list of _just_ the names\nof the authors?\n\n```elixir\n:xmerl_xpath.string('/*/Paper[@refNum=\"2\"]/Authors/Author', xml)\n```\n\nThis will get you the tags. To be fair, you can just index into the tuples to\nget the values you want. But that's probably a little brittle. What makes more\nsense is to use the `string()` function in the XPath:\n\n```elixir\n:xmerl_xpath.string('string(/*/Paper[@refNum=\"2\"]/Authors/Author/@name)', xml)\n```\n\nBut if you try this, you only get back one name. We can combine some of what\nwe've learned though to enumerate and get the names back easily:\n\n```elixir\n:xmerl_xpath.string('/*/Paper[@refNum=\"2\"]/Authors/Author/@name', xml)\n  |> Enum.map(fn el ->\n    :xmerl_xpath.string('string(/*)', el)\n  end)\n```\n\nWhich gives us:\n\n```elixir\n[\n  {:xmlObj, :string, 'A. Skywalker'},\n  {:xmlObj, :string, 'O. Kenobi'},\n  {:xmlObj, :string, 'A. Tano'}\n]\n```\n\nThis works because we get the exact node that we want, then we iterate over the\ngroup and simply apply the `string` function to that node. Using `/*` is\nnecessary because the node is now technically the root. This allows us to avoid\nneeding to use indices to access items.\n\nAnother common operation you might like to perform is to find something\nconditionally. For example, all the papers that were authored by a specific\nindividual. This is somewhat trickier:\n\n```elixir\n:xmerl_xpath.string('string(/*/Paper[descendant::Authors/Author[@name=\"L. Skywalker\"]]/Title)', xml)\n```\n\nHere we're using the conditional selection `Node[criterion]`. This would allow\nus to select e.g. all the nodes with a specific property. Then we use the\n`descendant::` qualifier to define the criterion as being on the children of the\npaper. Calling string on the title merely retrieves the desired datum.\n\nFor a more complex condition, let's say that we want to select all the papers\nthat were authored by A. Skywalker, either in name or in aka:\n\n```elixir\ncondition_1 = '/*/Paper[descendant::Authors/Author[@name=\"A. Skywalker\"]]/Title'\ncondition_2 = '/*/Paper[descendant::Authors/Author[@aka=\"A. Skywalker\"]]/Title'\n:xmerl_xpath.string('#{condition_1} | #{condition_2}', xml)\n```\n\nThis can again be used with the mapping and string selection to get the titles\nin a friendlier to access format, ala:\n\n```elixir\n:xmerl_xpath.string('#{condition_1} | #{condition_2}', xml)\n    |> Enum.map(fn el ->\n        :xmerl_xpath.string('string(/*)', el)\n    end)\n```\n\n#","ref":"use_xml_and_xpath.html#querying-paths-with-erlang","title":"Querying Paths with Erlang - How To Work With XML","type":"extras"},{"doc":"This is a really quick survey of the experimentation that I did to get some\nuseful XPath queries written. Hopefully these are helpful to you in dealing with\nany XML that is returned.\n\nIf you find that you want to do something and need a reference, then I can\nrecommend the [specification](https://www.w3.org/TR/2017/REC-xpath-31-20170321/#id-path-expressions) for XPath.\n\nIt's a little dense, but you should be able to use the reference to come up with\nanything that can be done.\n\n#","ref":"use_xml_and_xpath.html#conclusion","title":"Conclusion - How To Work With XML","type":"extras"},{"doc":"https://en.wikipedia.org/wiki/XML#Pull_parsing\nhttps://www.w3.org/TR/2008/REC-xml-20081126\nhttps://www.erlang.org/doc/man/xmerl\nhttps://starwars.fandom.com/wiki/Main_Page\nhttps://www.w3.org/TR/2017/REC-xpath-31-20170321/","ref":"use_xml_and_xpath.html#references","title":"References - How To Work With XML","type":"extras"}]}